import java.util.Vector;
import java.util.Enumeration;

class CandidateRecord {
    private String name;
    private int votes;
    
    public CandidateRecord (String n) {
	name = n;
	votes = 0;
    }
    
    public void recordVote()
    // MODIFIES: this
    // EFFECTS: Adds one vote to the vote count for this.  
    //    If this_pre = < n, c > then this_post = < n, c + 1 >.
    {
	votes++;
    }
    
    public String getName() {
	return name;
    }
    
    public int getVotes() {
	return votes;
    }
}

class OfficeResult {
    // OVERVIEW: A mutable datatype that represents the result for one
    //    race. 
    
    private String office;
    private Vector candidates; // A vector of CandidateRecord objects
    
    public OfficeResult(String o) {
	office = o;
	candidates = new Vector();
    }
    
    public void addCandidate(String c)
	// REQUIRES: c does not match the name of a candidate in this.
	// EFFECTS: Adds the candidate c to this with no votes.
	//      If this_pre = 
	//           < office, [ < candidate_0, votes_0 >, ..., < candidate_n-1, votes_n-1 > ]
	//      then this_post = 
	//           < office, [ < candidate_0, votes_0 >, ..., < candidate_n-1, votes_n-1 >, < c, 0 > ]
    {
	candidates.add(new CandidateRecord(c));
    }

    private CandidateRecord getRecord(String c)
    // EFFECTS: If c matches the name of a candidate in this, returns 
    //     the CandidateRecord in this.candidates whos name matches c.
    {
	for (int i = 0; i < candidates.size(); i++) {
	    CandidateRecord cr = (CandidateRecord) candidates.get(i);
	    if (cr.getName().equals(c)) {
		return cr;
	    }
	}
	
	throw new RuntimeException ("Invalid Candidate");
    }
    
    public String getOffice() {
	return office;
    }
    
    public void recordVote(String c) {
	// REQUIRES: c matches the name of a candidate in this.
	getRecord(c).recordVote();
    }
    
    public int getVotes(String c) {
	// REQUIRES: c matches the name of a candidate in this.
	return getRecord(c).getVotes();
    }
    
    public Vector getCandidates() {
	Vector res = new Vector ();
	for (Enumeration cands = candidates.elements (); cands.hasMoreElements (); ) {
	    CandidateRecord r = (CandidateRecord) cands.nextElement ();
	    res.add (r.getName());
	}
	return res;
    }
}

public class ElectionResults {
    // OVERVIEW: ElectionResults is a mutable datatype for recording the
    //    results of an election.  A typical ElectionResults is:
    //
    //        [ < office_0, results_0 >, < office_1, results_1 >, ..., < office_n-1, results_n-1 > ]
    //
    //    where office_i is the name of the ith office on the ballot and
    //
    //        results_i = [ < candidate_i_0, votes_i_0 > , ..., < candidate_i_m-1, votes_i_m-1 > ]
    //
    //     is a list of the candidates for office_i and the number of
    //     votes they received.
    //
    //     For example, 
    //
    //        [ < "Sheriff", [ < "Officer Krispy", 2043 >, < "Alice Enforcer", 3024 > ] >,
    //          < "Dog Catcher", [ < "Spot", 23 >, < "Muffy", 12 > ] > ]
    //      
    //     describes an election with two offices.  
    //
    
    private Vector results; // A vector of OfficeResult objects
    
    public ElectionResults() {
	results = new Vector();
    }
    
    public void addOffice(Office o)
    // REQUIRES: this does not contain an office with the same name
    //              as o.
    // EFFECTS: Adds the office o to this election.
    {
	OfficeResult or = new OfficeResult(o.getName());
	for (Enumeration candidates = o.candidates(); candidates.hasMoreElements(); ) {
	    or.addCandidate((String) candidates.nextElement());
	}
	
	results.add(or);
    }
    
    private OfficeResult lookupOffice(String s)
    // EFFECTS: If s is the name of an office in this election,
    //             returns the OfficeResult object associated with s.
    {
	for (Enumeration offices = results.elements(); offices.hasMoreElements(); ) {
	    OfficeResult or = (OfficeResult) offices.nextElement();
	    if (or.getOffice().equals(s)) {
		return or;
	    }
	}
	
	throw new RuntimeException ("No such office");
    }
    
    public void recordVote(String office, String candidate) {
	OfficeResult ofr = lookupOffice(office);
	ofr.recordVote(candidate);
    }

    public int getVotes(String office, String candidate) {
	OfficeResult ofr = lookupOffice(office);
	return ofr.getVotes(candidate);
    }
    
    public String getWinner(String office)
    // EFFECTS: Returns the name of the candidate for office
    //    who received more votes than every other candidate.
    //    If no such candidate exists, throws an exception.
    //    For example, if this_pre =
    //        [ < "Sheriff", [ < "Officer Krispy", 3 >, < "Alice Enforcer", 12 > ] >,
    //          < "Dog Catcher", [ < "Sarge", 4 >, < "Mooch", 4 > ] > ]
    //    getWinner ("Sheriff") returns "Alice Enforcer" and
    //    getWinner ("Dog Catcher") throws NoWinnerException.
    {
	OfficeResult or = lookupOffice(office);
	String winner = null;
	int maxvotes = 0;
	
	Vector candidates = or.getCandidates();
	for (Enumeration candenum = candidates.elements(); candenum.hasMoreElements(); ) {
	    String candidate = (String) candenum.nextElement();
	    int cvotes = getVotes(office, candidate);
	    
	    if (cvotes == maxvotes) {
		winner = null;
	    } else if (cvotes > maxvotes) {
		maxvotes = cvotes;
		winner = candidate;
	    }
	}
	
	if (winner != null) {
	    return winner;
	} else {
	    throw new RuntimeException("NoWinner");
	}
    }
}












